@@ -40,6 +40,11 @@ ipsecr.fit <- function (
     #################################################
     
     trps <- traps(capthist)
+    
+    if (!is.null(covariates(mask))) {
+        trps <- addCovariates(trps, mask)
+    }
+    
     #------------------------------------------
     # optionally replace detector type
     if (!is.null(details$newdetector)) {
@@ -47,7 +52,7 @@ ipsecr.fit <- function (
         detectortype <- details$newdetector       
         if (ms(trps)) for (j in 1:length(trps)) detector(trps[[j]]) <- detectortype
         else  detector(trps) <- detectortype
-    }
+    } 
     else {
         if (ms(trps)) detectortype <- unlist(sapply(trps, detector))[1]   ## assume all same
         else detectortype <- detector(trps)[1]
@@ -78,17 +83,16 @@ ipsecr.fit <- function (
         boxsize2     = 0.05, 
         boxtype      = 'absolute',
         centre       = 3,
-        dev.max      = 0.002, 
+        dev.max      = 0.002,
         var.nsim     = 2000,
+        keep.sim     = FALSE,
         min.nsim     = 20,
-        max.nsim     = 2000, 
+        max.nsim     = 200, 
         min.nbox     = 2, 
         max.nbox     = 5, 
         max.ntries   = 2,
         distribution = 'poisson',
-        even         = FALSE,
         binomN       = 0,               ## Poisson counts
-        param        = 0,
         ignoreusage  = FALSE,
         ignorenontarget = FALSE,
         nontargettype = 'exclusive',
@@ -100,7 +104,9 @@ ipsecr.fit <- function (
         popmethod    = 'internal',
         Nmax         = 1e4,
         factorial    = 'full',
-        FrF2args     = NULL
+        FrF2args     = NULL,
+        YonX         = TRUE,
+        param        = 0               ## needed by 'secr'
     )
     if (any(!(names(details) %in% names(defaultdetails)))) {
         stop ("details list includes invalid name")
@@ -119,6 +125,10 @@ ipsecr.fit <- function (
     details$factorial<- match.arg(details$factorial, choices=c('full','fractional'))
     details$verbose <- verbose
     
+    # expand dev.max to max.nbox vector and allow two-tier precision
+    dev.max <- rep(details$dev.max[1], length.out = details$max.nbox)
+    if (length(details$dev.max)>1) dev.max[2:details$max.nbox] <- details$dev.max[2]
+    
     #################################################
     # nontarget model 
     #################################################
@@ -147,7 +157,7 @@ ipsecr.fit <- function (
         cellarea <- sapply(mask, attr, 'area')
         trapmeanSD <- lapply(trps, getMeanSD)
         nontarget <- sapply(capthist, attr, 'nontarget', exact = TRUE)
-    }
+    } 
     else {
         nsessions <- 1
         noccasions <- ncol(capthist)
@@ -155,9 +165,7 @@ ipsecr.fit <- function (
         trapmeanSD <- getMeanSD(trps)
         nontarget <- attr(capthist, 'nontarget', exact = TRUE)
     }
-    
-    
-    
+
     #################################################
     ## optional data check
     #################################################
@@ -177,13 +185,13 @@ ipsecr.fit <- function (
     #################################################
     ## nontarget interference 
     #################################################
-    
+   
     modelnontarget <- length(unlist(nontarget))>0 && !details$ignorenontarget
     ## check
     if (modelnontarget) {
         trapdesigndata <- D.designdata(trps, model$NT, 1, 
             sessionlevels, sessioncov, trapmeanSD)
-    }
+    } 
     else {
         trapdesigndata <- NULL
         model$NT <- NULL
@@ -197,7 +205,15 @@ ipsecr.fit <- function (
     model <- stdform (model)  ## named, no LHS
     model <- updatemodel(model, detectfn, 14:20, 'g0', 'lambda0')
     
-    if (any(sapply(model[-1], "!=", ~1))) stop ("not ready for varying detection")
+    detmodels <- names(model) %in% c('g0','lambda0','sigma')
+    # if (any(sapply(model[detmodels], "!=", ~1))) stop ("not ready for varying detection")
+    notOK <- function(model) {
+        sessvars <- names(sessioncov)
+        maskvars <- names(covariates(if (ms(mask)) mask[[1]] else mask))  
+        !all(all.vars(model) %in% c("session", "Session", "x", "y", sessvars, maskvars))
+    }
+    
+    if (any(sapply(model[detmodels], notOK))) stop ("detection model includes unavailable variables")
     
     fnames <- names(fixed)
     
@@ -245,18 +261,6 @@ ipsecr.fit <- function (
     link <- replace (defaultlink, names(link), link)
     link[!(names(link) %in% pnames)] <- NULL
     
-    ##############################################
-    # Prepare detection design matrices and lookup
-    ##############################################
-    # memo ('Preparing detection design matrices', verbose)
-    design <- secr.design.MS (capthist, model, timecov, sessioncov, NULL, NULL,
-        NULL, ignoreusage = details$ignoreusage, naive = FALSE,
-        CL = FALSE, contrasts = details$contrasts)
-    design0 <- design   # for now
-    # design0 <- secr.design.MS (capthist, model, timecov, sessioncov, NULL, NULL,
-    #     NULL, ignoreusage = details$ignoreusage, naive = TRUE,
-    #     CL = FALSE, contrasts = details$contrasts)
-    
     ############################################
     # Prepare density design matrix
     ############################################
@@ -266,12 +270,13 @@ ipsecr.fit <- function (
         designD <- matrix(nrow = 0, ncol = 0)
         attr(designD, 'dimD') <- NA
         nDensityParameters <- integer(0)
-    }
+    } 
     else {
         memo ('Preparing density design matrix', verbose)
         temp <- D.designdata( mask, model$D, 1, sessionlevels, sessioncov)
-        designD <- general.model.matrix(model$D, data = temp, gamsmth = NULL,
-            contrasts = details$contrasts)
+        designD <- model.matrix(model$D, data = temp, contrasts.arg = details$contrasts)
+        rownames (designD) <- NULL
+        
         attr(designD, 'dimD') <- attr(temp, 'dimD')
         Dnames <- colnames(designD)
         nDensityParameters <- length(Dnames)
@@ -289,13 +294,13 @@ ipsecr.fit <- function (
     if (!NT.modelled) {
         designNT <- matrix(nrow = 0, ncol = 0)
         attr(designNT, 'dimD') <- NA
-        nNTParameters <- integer(0)
-    }
+        nNTParameters <- 0  # integer(0)
+    } 
     else {
         memo ('Preparing NT design matrix', verbose)
         temp <- D.designdata( trps, model$NT, 1, sessionlevels, sessioncov, trapmeanSD)
-        designNT <- general.model.matrix(model$NT, data = temp, gamsmth = NULL,
-            contrasts = details$contrasts)
+        designNT <- model.matrix(model$NT, data = temp, contrasts = details$contrasts)
+        rownames(designNT) <- NULL
         attr(designNT, 'dimD') <- attr(temp, 'dimD')
         NTnames <- colnames(designNT)
         nNTParameters <- length(NTnames)
@@ -309,13 +314,26 @@ ipsecr.fit <- function (
     ############################################
     # Parameter mapping (general)
     ############################################
-    detectparnames <- names(design$designMatrices) %in% c('g0','lambda0','sigma')
-    nDetectionParameters <- sapply(design$designMatrices[detectparnames], ncol)
+    popn <- simpop(mask, D=1, N = 10, details) # throw-away popn for parameter counting
+    detbetanames <- detBetaNames(popn, model, detectfn, sessionlevels, fixed, details)
+    nDetectionParameters <- sapply(detbetanames, length)
+    
     np <- c(D = nDensityParameters, nDetectionParameters, NT = nNTParameters)
     NP <- sum(np)
     parindx <- split(1:NP, rep(1:length(np), np))
     names(parindx) <- names(np)[np>0]
     
+    ############################################
+    # Variable names (general)
+    ############################################
+    realnames <- names(model)
+    ## coefficients for D precede all others
+    betanames <- c(paste('D', Dnames, sep='.'), unlist(detbetanames))
+    if (nNTParameters>0) {
+        betanames <- c(betanames, paste('NT', NTnames, sep='.'))
+    }
+    betanames <- sub('..(Intercept))','',betanames)
+    
     ###########################################
     # setup boxes
     ###########################################
@@ -343,17 +361,11 @@ ipsecr.fit <- function (
         N <- switch(tolower(Ndist), poisson = rpois(1, N), fixed = round(N), NA)
         
         # cannot simulate zero animals, so return NA for predicted
-        if (is.na(N) || any(N<=0) || any(N>details$Nmax)) return(rep(NA, NP))
+        if (any(is.na(N)) || any(N<=0) || any(N>details$Nmax)) return(rep(NA, NP))
         
-        # detectpar is list of named parameters of detectfn
-        # not yet modelled using design, design0: assumes 1:1
-        detectparnames <- names(parindx) %in% c('g0','lambda0','sigma')
-        betalist <- lapply(parindx[detectparnames], function(x) unname(beta[x]))
-        detectpar <- mapply(untransform, betalist, link[detectparnames], SIMPLIFY = FALSE)
-
         if (modelnontarget) {
             NT <- getD('NT', designNT, beta, trps, parindx, link, fixed, nsessions)
-        }
+        } 
         else {
             NT <- NULL
         }
@@ -363,10 +375,10 @@ ipsecr.fit <- function (
             #------------------------------------------------
             # generate population
             if (is.function(details$popmethod)) {   # user
-                popn <- details$popmethod(mask, D, N)
+                popn <- details$popmethod(mask = mask, D = D, N = N, details = details)
             }
             else if (details$popmethod == 'internal') {   # C++
-                popn <- simpop(mask, D, N, distribution)
+                popn <- simpop(mask, D, N, details)
             }
             else if (details$popmethod == "sim.popn") {
                 popn <- sim.popn (D = as.data.frame(D), core = mask, 
@@ -377,18 +389,34 @@ ipsecr.fit <- function (
                 warning ("ipsecr.fit: no animals in simulated population", call. = FALSE)
                 return(rep(NA, NP))
             }
+            #-----------------------------------------------------
+            # session-specific detection parameter matrix/matrices
+            scalepopn <- function (popn, mask) {
+                if (ms(popn)) {
+                    out <- mapply(scalepopn, popn, mask, SIMPLIFY = FALSE)
+                    class(out) <- class(popn)
+                    out
+                } 
+                else {
+                    meanSD <- attr(mask,'meanSD')
+                    popn[,] <- scale(popn, meanSD[1,], meanSD[2,])
+                    popn
+                }
+            }
+            popn2 <- scalepopn(popn, mask)
+            detparmat <- getDetParMat (popn2, model, detectfn, beta, parindx, link, fixed, details, sessionlevels)
+            #-----------------------------------------------------
 
-            #------------------------------------------------
             # sample from population
             if (is.function(details$CHmethod)) {   # user
-                ch <- details$CHmethod(trps, popn, detectfn, detectpar, noccasions, details)
+                ch <- details$CHmethod(trps, popn, detectfn, detparmat, noccasions, NT, details)
             }
             else if (details$CHmethod == 'internal') {   # C++
-                ch <- simCH(trps, popn, detectfn, detectpar, NT, noccasions, details)
+                ch <- simCH(trps, popn, detectfn, detparmat, noccasions, NT, details)
             }
             else if (details$CHmethod == 'sim.capthist') {   
                 if (modelnontarget) stop ("sim.capthist does not simulate nontarget detections")
-                ch <- sim.capthist(trps, popn, detectfn, detectpar, noccasions, nsessions, 
+                ch <- sim.capthist(trps, popn, detectfn, as.list(detparmat[1,]), noccasions, nsessions, 
                     renumber = FALSE)
             }
             # check valid
@@ -396,7 +424,7 @@ ipsecr.fit <- function (
             r <- if (nsessions == 1) sum(abs(ch)>0) else sapply(ch, function(x) sum(abs(x)>0))
             if (any(n==0)) {
                 warning ("ipsecr.fit: no captures in simulation", call. = FALSE)
-            }
+            } 
             else {
                 if (any((r - n) < 1))
                     warning ("ipsecr.fit: no re-captures in simulation", call. = FALSE)
@@ -412,7 +440,7 @@ ipsecr.fit <- function (
                 cat ('saving ch to ch.RDS\n')
                 saveRDS(ch, file='ch.RDS')
                 cat('N ', N, '\n')
-                cat('detectpar', unlist(detectpar), '\n')
+                cat('detparmat', unlist(detparmat), '\n')
                 cat('detectfn', detectfn, '\n')
                 print(summary(ch))
                 cat('\nproxyfn\n')
@@ -434,7 +462,7 @@ ipsecr.fit <- function (
             warning ("ipsecr.fit: no successful simulation after ", details$max.ntries,
                 " attempts", call. = FALSE)
             return (rep(NA, NP))
-        }
+        } 
         else {
             if (attempts > 1)
                 warning ("ipsecr.fit: simulation repeated", call. = FALSE)
@@ -455,8 +483,8 @@ ipsecr.fit <- function (
     ##########################################
     y <- proxyfn(capthist, model = model, trapdesigndata = trapdesigndata, ...)
     if (length(y) != NP)
-        stop ("need one proxy for each parameter ",
-            paste(pnames, collapse=" "))
+        stop ("need one proxy for each coefficient ",
+            paste(betanames, collapse=" "))
 
     ##########################################
     ## starting values
@@ -483,16 +511,24 @@ ipsecr.fit <- function (
         NP <- length(start)
     }
     
-    ############################################
-    # Variable names (general)
-    ############################################
-    betanames <- unlist(sapply(design$designMatrices, colnames))
-    names(betanames) <- NULL
-    realnames <- names(model)
-    ## coefficients for D precede all others
-    betanames <- c(paste('D', Dnames, sep='.'), betanames)
-    betanames <- sub('..(Intercept))','',betanames)
+    ##########################
+    # set random seed
+    # (from simulate.lm)
+    ##########################
 
+    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
+        runif(1)
+    }
+    if (is.null(seed)) {
+        RNGstate <- get(".Random.seed", envir = .GlobalEnv)
+    }
+    else {
+        R.seed <- get(".Random.seed", envir = .GlobalEnv)
+        set.seed(seed)
+        RNGstate <- structure(seed, kind = as.list(RNGkind()))
+        on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
+    }
+    
     ###########################################
     # cluster for parallel processing
     ###########################################
@@ -502,7 +538,7 @@ ipsecr.fit <- function (
         if(.Platform$OS.type == "unix") {
             # clust <- makeCluster(ncores, type = "FORK", outfile = "clusterlogfile.txt")
             clust <- makeCluster(ncores, type = "FORK")
-        }
+        } 
         else {
             # clust <- makeCluster(ncores, type = "PSOCK", outfile = "clusterlogfile.txt")
             clust <- makeCluster(ncores, type = "PSOCK")
@@ -510,15 +546,20 @@ ipsecr.fit <- function (
                 "mask", "trps", "link", "fixed", "details", 
                 "detectfn", "noccasions", "nsessions", "proxyfn",
                 "model", "trapdesigndata", "parindx", 
-                "designD", "designNT", "modelnontarget", "cellarea"), 
+                "designD", "designNT", "modelnontarget", "cellarea"
+                # following are exported only during testing
+                # ,"getD", "untransform", "simpop", "simCH", "ms", "getDetParMat", 
+                # "parnames", "getDetDesignData", "covariates", "invlogit", 
+                # "usage", "detector", "CHcpp", "traps<-", "MS.capthist"
+                ), 
                 environment())
         }
         on.exit(stopCluster(clust))
         clusterSetRNGStream(clust, seed)
-    }
+    } 
     else {
         clust <- NULL
-        set.seed(seed)
+        #set.seed(seed)
     }
 
     ####################################################################
@@ -528,15 +569,21 @@ ipsecr.fit <- function (
     
     beta <- start
     ip.nsim <- numeric(0)
+    if (details$keep.sim) {
+      simulations <- vector('list')
+      parameters  <- vector('list')
+    }
+    
     for (m in 1:details$max.nbox) {
         if (verbose) {
             cat('\nFitting box', m, '...  \n')
         }
+        inbox <- FALSE
         boxsize <- if (m == 1) boxsize1 else boxsize2
         if (details$boxtype == 'relative') {
             vertices <- sweep (1 + outer(c(-1,1), boxsize), MARGIN = 2,
                 FUN = '*', STATS = beta)
-        }
+        } 
         else {
             vertices <- sweep (outer(c(-1,1), boxsize), MARGIN = 2,
                 FUN = '+', STATS = beta)
@@ -555,7 +602,7 @@ ipsecr.fit <- function (
             designbeta <- as.matrix(expand.grid (as.list(vertices)))
             centrepoints <- matrix(beta, nrow = details$centre, ncol = NP, byrow = T)
             designbeta <- rbind(designbeta, centrepoints)
-        }
+        } 
         else {
             if (!requireNamespace('FrF2', quietly = TRUE))
                 stop ('package FrF2 required for fractional factorial')
@@ -568,23 +615,16 @@ ipsecr.fit <- function (
             base <- sweep(base, MARGIN = 2, STATS = boxsize, FUN = '*')
             if (details$boxtype == "absolute") {
                 designbeta <- sweep(base, MARGIN = 2, STATS = beta, FUN = '+')
-            }
+            } 
             else {
                 designbeta <- sweep(base, MARGIN = 2, STATS = beta, FUN = '*')
             }
         }
         
-        # check number of simulations
-        designpoints <- nrow(designbeta)
-        if (2 * designpoints * details$min.nsim > details$max.nsim) {
-            stop("max.nsim = ", details$max.nsim, 
-                " does not allow two boxes with designpoints = ", designpoints, 
-                " and min.nsim = ", details$min.nsim)
-        }
-        
+        ndesignpoints <- nrow(designbeta)
         sim <- NULL
         alldesignbeta <- NULL
-        tempdistn <- if (details$even) 'even' else 'binomial'
+        tempdistn <- if (details$distribution == 'even') 'even' else 'binomial'
         basedesign <- designbeta[rep(1:nrow(designbeta), details$min.nsim),]
         # accumulate simulations until reach precision target or exceed max.nsim
         tries <- 0
@@ -595,7 +635,7 @@ ipsecr.fit <- function (
                 newsim <- parRapply(clust, basedesign, simfn, 
                     distribution = tempdistn, ...)
                 newsim <- t(matrix(newsim, ncol = nrow(basedesign)))
-            }
+            } 
             else {
                 newsim <- t(apply(basedesign, 1, simfn, 
                     distribution = tempdistn, ...))
@@ -607,25 +647,42 @@ ipsecr.fit <- function (
             }
             sim <- rbind(sim, newsim[OK,])
             alldesignbeta <- rbind(alldesignbeta,basedesign[OK,])
-            sim.lm <- lm ( sim ~ alldesignbeta )   # proxy ~ link(param)
+            
+            if (details$YonX) {
+                sim.lm <- lm ( sim ~ alldesignbeta )  # proxy ~ link(param)
+            } 
+            else {
+                warning ("regression of X on Y does not work in ipsecr 1.2.0")
+                sim.lm <- lm ( alldesignbeta ~ sim)  # link(param) ~ proxy
+            }
+
+            # break if have exceeded allowed simulations
+            if ((nrow(alldesignbeta) > (details$max.nsim * ndesignpoints))) break
+            
+            # alternatively, determine whether sufficient precision achieved
+            #--------------------------------------------------------------------
             sum.sim.lm <- summary(sim.lm)
             code <- 0
             if (details$boxtype == 'absolute') {
                 dev <- sapply(sum.sim.lm, function(x) x$sigma) / sqrt(nrow(sim))
-            }
+            } 
             else {
                 dev <- sapply(sum.sim.lm, function(x) x$sigma) / y / sqrt(nrow(sim))
             }
-            # break if have exceeded allowed simulations
-            if ((nrow(alldesignbeta) > details$max.nsim)) break
             # break if have achieved target precision
-            if (!is.null(dev) && !any(is.na(dev)) && all(dev <= details$dev.max)) break
+            if (!is.null(dev) && !any(is.na(dev)) && all(dev <= dev.max[m])) break
+            #-------------------------------------------------------------------
         }
         
         # simulations for this box
         ip.nsim <- c(ip.nsim, nrow(sim))  # 2022-06-14
         
-        faildev <- (dev > details$dev.max)
+        if (details$keep.sim) {
+          simulations[[m]] <- sim
+          parameters[[m]] <- alldesignbeta
+        }
+        
+        faildev <- (dev > dev.max[m])
         if (any(faildev)) {
             # namestring <-  paste(names(y)[faildev][1:2], collapse = ', ')
             # if (sum(faildev)>1) namestring <- paste('proxies', namestring)
@@ -634,7 +691,7 @@ ipsecr.fit <- function (
             crit <- switch(details$boxtype, absolute = 'SE', relative = 'RSE')
             memo(paste0("simulations for box ", m, " did not reach target precision:"), verbose)
             if (verbose) {
-                out <- data.frame(proxy = names(y), dev = dev, target = details$dev.max)
+                out <- data.frame(proxy = names(y), dev = dev, target = dev.max[m])
                 names(out) <- c('Proxy', crit, paste0('target.', crit))
                 print(out[faildev,], row.names = FALSE)
             }
@@ -646,35 +703,44 @@ ipsecr.fit <- function (
             # if (code == 4) warning ("exceeded maximum allowable replicates ",
             #     "without achieving precision better than 'dev.max'")
             # if (code == 5) warning ("ipsecr.fit: invalid lm")
-        }
+        } 
         else {
             B <- coef(sim.lm)[-1,]
-            # B <- solve(t(B))  ## invert
-            B <- try(MASS::ginv(t(B)), silent = TRUE)   ## 2022-05-15
-            if (inherits(B, 'try-error')) {
-                code <- 5
-                beta <- rep(NA, NP)
-            }
-            else {
+            if (details$YonX) {
                 lambda <- coef(sim.lm)[1,]   ## intercepts
-                beta <- as.numeric(B %*% matrix((y - lambda), ncol = 1))
-                if (all(sapply(1:NP, within)) && (m >= details$min.nbox)) break
+                B <- try(MASS::ginv(t(B)), silent = TRUE)   ## 2022-05-15
+                if (inherits(B, 'try-error')) {
+                    code <- 5
+                    beta <- rep(NA, NP)
+                } 
+                else {
+                    beta <- as.numeric(B %*% matrix(y - lambda, ncol = 1))
+                }
+            } 
+            else {
+                beta <- as.numeric(matrix(y, nrow=1) %*% coef(sim.lm)[-1,] + coef(sim.lm)[1,])
             }
+            inbox <- all(sapply(1:NP, within))
+            if (inbox && (m >= details$min.nbox)) break
         }
     }    # end of loop over boxes
     ####################################################################
+    if (verbose) {
+        cat('Simulations per vertex, by box : ', 
+            paste(round(ip.nsim/ndesignpoints,1), collapse = ', '), '\n')
+    }
     
+    ####################################################################
     if (code == 0) {
-        if (!all(sapply(1:NP, within))) {
+        if (!inbox) {
             warning ("solution not found after ", details$max.nbox, " attempts")
             code <- 2
-        }
+        } 
         else {
             code <- 1
         }
     }
     ####################################################################
-    
     if (details$var.nsim>1 && code == 1) {
         if (verbose) {
             cat('\nSimulating for variance ...\n')
@@ -689,7 +755,7 @@ ipsecr.fit <- function (
             newsim <- parRapply(clust, vardesign, simfn, 
                 distribution = details$distribution, ...)
             newsim <- t(matrix(newsim, ncol = nrow(vardesign)))
-        }
+        } 
         else {
             newsim <- t(apply(vardesign,1,simfn, 
                 distribution = details$distribution, ...))
@@ -701,30 +767,38 @@ ipsecr.fit <- function (
         }
         newsim <- newsim[OK,]
         V <- var(newsim)  ## additional simulations for var-covar matrix
-        vcov <- B %*% V %*% t(B)
+        if (details$YonX) {
+            vcov <- B %*% V %*% t(B)
+        } 
+        else {
+            # NOT RIGHT
+            # vcov <- coef(sim.lm)[-1,] %*% V %*% t(coef(sim.lm)[-1,])
+            warning ("variances not available for XonY")
+            vcov <- matrix(nrow = NP, ncol = NP)
+        }
         
         ## compare estimates to parametric bootstrap
-        n <- apply(newsim, 2, function(x) sum(!is.na(x)))
-        ymean <- apply(newsim, 2, mean, na.rm=T)
-        yse <- apply(newsim, 2, function(x) sd(x, na.rm=T) / sum(!is.na(x)))
-        
-        bootstrap <- data.frame (target = y, nsim = n, simulated = ymean,
-            SE.simulated = yse)
+        nsim <- apply(newsim, 2, function(x) sum(!is.na(x)))
+        ymean <- apply(newsim, 2, mean, na.rm = T)
+        yse <- apply(newsim, 2, function(x) sd(x, na.rm = T) / sum(!is.na(x)))
+        yq <- apply(newsim, 2, function(x) quantile(x, na.rm = T, probs = c(0.025,0.5,0.975)))
+
+        bootstrap <- data.frame (target = y, nsim = nsim, simulated = ymean,
+            SE.simulated = yse, q025 = yq[1,], median = yq[2,], q975 = yq[3,])
         
         ## biasest not reported, yet
-        yest <- as.numeric(B %*% matrix((ymean - lambda), ncol = 1))
-        biasest <- data.frame (estimate = 100 * (beta - yest) / yest,
-            SE = 100 * (beta - yest) / yest)
+        # yest <- as.numeric(B %*% matrix((ymean - lambda), ncol = 1))
+        # biasest <- data.frame (estimate = 100 * (beta - yest) / yest,
+        #     SE = 100 * (beta - yest) / yest)
         
-    }
+    } 
     else {
         vcov <- matrix(nrow = NP, ncol = NP)
         bootstrap <- NA
     }
+
     dimnames(vcov) <- list(betanames, betanames)
     
-    desc <- packageDescription("ipsecr")  ## for version number
-
     output <- list(
         call = cl,
         capthist = capthist,
@@ -740,8 +814,6 @@ ipsecr.fit <- function (
         details = details,
         designD = designD,   
         trapdesigndata = trapdesigndata, 
-        design = design,
-        design0 = design0,
         parindx = parindx,
         vars = vars,
         betanames = betanames,
@@ -749,13 +821,17 @@ ipsecr.fit <- function (
         code = code,                # success of fit
         beta = beta,
         beta.vcv = vcov,
-        designbeta = designbeta,   # last 
+        designbeta = designbeta,    # last 
+        sim.lm = sim.lm,            # added 2022-08-02
         ip.nsim = ip.nsim,
         var.nsim.OK = if(details$var.nsim>1) sum(OK) else NA,
+        simulations = if (details$keep.sim) simulations else NULL,
+        parameters = if (details$keep.sim) parameters else NULL,
         variance.bootstrap = bootstrap,
-        version = desc$Version,
+        version = packageDescription("ipsecr")$Version,
         starttime = starttime,
-        proctime = as.numeric((proc.time() - ptm)[3])
+        proctime = as.numeric((proc.time() - ptm)[3]),
+        seed = RNGstate
     )
     class(output) <- 'ipsecr'
     memo(paste('Completed in ', round(output$proctime,2), ' seconds at ',
